# function to compute linear interpolation of VE array
function calcve(rpm) {
	x = int(rpm / 500);
	vl = ve[x];
	x = x + 1;
	vh = ve[x];
	x = rpm % 500 / 500.;
	v = vl + ((vh - vl) * x);
	return v;
}

BEGIN {
	FS= ","; # CSV file

	# VE table of Renesis as measured and posted by Hymee
	# http://www.rx8club.com/showthread.php?t=33370
	ve[0] = 1.;    #    0RPM No data
	ve[1] = 1.;    #  500RPM No data
	ve[2] = 1.;    # 1000RPM No data
	ve[3] = 1.;    # 1500RPM No data
	ve[4] = 1.;    # 2000RPM No data
	ve[5] = .88;   # 2500RPM
	ve[6] = .83;   # 3000RPM
	ve[7] = .85;   # 3500RPM
	ve[8] = .84;   # 4000RPM
	ve[9] = .82;   # 4500RPM
	ve[10] = .89;  # 5000RPM
	ve[11] = .93;  # 5500RPM
	ve[12] = 1.01; # 6000RPM
	ve[13] = .96;  # 6500RPM
	ve[14] = 1.01; # 7000RPM
	ve[15] = .98;  # 7500RPM
	ve[16] = .98;  # 8000RPM
	ve[17] = .92;  # 8500RPM
	ve[18] = .89;  # 9000RPM

	# Density of air in grams/liter
	# 85degF, 40% humidity, sea level
	density = 1.1596;

	# Print header for CSV file
	printf "time,throttle,boost,air pressure at BDC after port closure,FI maf measured,WOT maf NA,rpm,natural VE\n"
}
$4 != "" { maf = $4; }
$9 != "" { 
	# Have to predict exact RPM since mysql's logger doesn't give
	# RPM and MAF reading in same time instant.  Base it on calculated
	# acceleration between last two readings.  This is mostly
	# inconsequential in the calculation, but good to be pedantic. 
	rpm_last = rpm;
	rpm = $9; 
	rpm_accel = rpm - rpm_last;
	rpm_p = rpm + (rpm_accel / 2);
	rps = (rpm_p+1e-9)/60.;  # revs per second
}
$5 != "" { throttle = $5; }
$2 != "" { 
	# measured grams of air per revolution
	gpr_fi = maf/rps; 

	# naturally aspirated grams of air per 1.308L revolution
	gpr_na = density * calcve(rpm_p) * 1.308; 

	# real pressure ratio turbo must generate in intake to force
	# mass of air to be ingested
	boost = (14.7 * gpr_fi / gpr_na) - 14.7;

	# real absolute pressure of air in 1.308L combustion chamber
	# after intake port closes
	boost2 = (14.7 * gpr_fi / (density * 1.308)) - 14.7;

	# MAF reading of NA renesis at WOT
	maf_na = gpr_na * rps;

	printf("%f,%f,%f,%f,%f,%f,%d,%f\n", $1, throttle, boost, boost2, maf, 
	  maf_na, rpm_p, calcve(rpm_p));  
}
